from lxml import etree
from lxml.etree import XMLSyntaxError

from concentrator.smev3.base.utils import get_order_id
from concentrator.smev3.event_service.helpers import (
    get_text_message_event_request)
from concentrator.smev3.service_types.kinder_conc_event import (
    EventServiceResponseType)
from kinder.webservice.smev3.utils.request_builder import BaseRequestBuilder


def handle_text_message_response(request, ext_result):
    """
    Возвращает code и message из ответа на запрос

    :param request: Объект TextMessageEventRequest
    :type request: TextMessageEventRequest
    :param ext_result: результат выполнения предыдущего обработчика точки
        расширения (в случае, если таковых несколько)
    :type ext_result: Any
    :return: Код и сообщение
    :rtype: tuple
    """

    node = etree.fromstring(request.response)
    response = EventServiceResponseType().build(node)

    return response.code, response.message


def get_text_message_event_builder(ext_result):
    """Возвращает класс билдера для TextMessageEventRequest

    :param ext_result: результат выполнения предыдущего обработчика точки
        расширения (в случае, если таковых несколько)
    :type ext_result: Any
    :return: Класс билдера
    :rtype: TextMessageEventRequestBuilder
    """

    class TextMessageEventRequestBuilder(BaseRequestBuilder):
        """Класс билдера для запроса отправки сообщений"""

        def build(self):
            """
            Метод построения запроса

            :return: Текст запроса
            :rtype: str
            """

            order_id = get_order_id(self.request.message_exchange.declaration)
            return get_text_message_event_request(order_id, self.request)

    return TextMessageEventRequestBuilder


def validate_text_message_response(response, ext_result):
    """
    Валидация eventServiceResponse

    :param response: Текст ответа на запрос
    :type response: str
    :param ext_result: результат выполнения предыдущего обработчика точки
        расширения (в случае, если таковых несколько)
    :type ext_result: Any
    :return: Есть ли ошибка в сообщении
    :rtype: bool
    """

    try:
        node = etree.fromstring(response)
        response = EventServiceResponseType().build(node)
    except XMLSyntaxError:
        return False
    else:
        return response.code or response.message or False
